"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var treeview_service_1 = require("./treeview.service");
var TreeViewComponent = (function () {
    function TreeViewComponent(treeViewService) {
        this.treeViewService = treeViewService;
        this.selectedRecord = new core_1.EventEmitter();
    }
    TreeViewComponent.prototype.ngOnInit = function () {
    };
    TreeViewComponent.prototype.ngAfterViewInit = function () {
        if (this.httpMethod && this.httpUrl) {
            this.treeViewService.fetchData(this, this.httpUrl, this.httpMethod);
        }
        else if (this.dataTableBindData) {
            this.setData(this.dataTableBindData);
        }
    };
    TreeViewComponent.prototype.ngOnChanges = function (change) {
        if (change['dataTableBindData']) {
            var data = change['dataTableBindData'].currentValue;
            if (data) {
                this.setData(data);
            }
        }
    };
    TreeViewComponent.prototype.setData = function (httpResponse) {
        var treedata = this.getResponseData(httpResponse);
        if (treedata) {
            this.data = treedata;
        }
    };
    TreeViewComponent.prototype.getResponseData = function (httpResponse) {
        var responsedata = httpResponse;
        var dr = this.dataReader.split(".");
        for (var ir = 0; ir < dr.length; ir++) {
            responsedata = responsedata[dr[ir]];
        }
        return responsedata;
    };
    TreeViewComponent.prototype.toggle = function (treeData) {
        if (!treeData.leaf)
            treeData.expanded = !treeData.expanded;
        if (treeData.lazy && treeData.children && treeData.children.length <= 0) {
            this.lazyNode = treeData;
            this.treeViewService.fetchLazyData(this, treeData.lazy.httpUrl, treeData.lazy.httpMethod);
        }
    };
    TreeViewComponent.prototype.setLazyData = function (httpResponse) {
        delete this.lazyNode.leaf;
        this.lazyNode['expanded'] = true;
        for (var di = 0; di < httpResponse.data.length; di++) {
            this.lazyNode.children.push(httpResponse.data[0]);
        }
    };
    TreeViewComponent.prototype.setSelectedRecord = function (treeData) {
        this.emitData(treeData);
    };
    TreeViewComponent.prototype.emitData = function (treeData) {
        this.selectedRecord.emit(JSON.parse(JSON.stringify(treeData)));
    };
    return TreeViewComponent;
}());
__decorate([
    core_1.Input(),
    __metadata("design:type", String)
], TreeViewComponent.prototype, "httpUrl", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", String)
], TreeViewComponent.prototype, "httpMethod", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", String)
], TreeViewComponent.prototype, "dataReader", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Object)
], TreeViewComponent.prototype, "dataTableBindData", void 0);
__decorate([
    core_1.Output(),
    __metadata("design:type", Object)
], TreeViewComponent.prototype, "selectedRecord", void 0);
TreeViewComponent = __decorate([
    core_1.Component({
        selector: 'tree-view',
        template: "<ul style=\"list-style-type: none;\">\n        <li style=\"cursor: pointer\" *ngFor=\"let treeData of data\">\n            <div >\n                <span [ngClass]=\"{'glyphicon glyphicon-minus': treeData.expanded, 'glyphicon glyphicon-plus': (!treeData.expanded && treeData.children)}\" (click)=\"toggle(treeData)\"> </span>\n                <img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\"><label style=\"cursor: pointer\" (click)=\"emitData(treeData)\">{{ treeData.text }}</label>\n            </div>\n            <div *ngIf=\"treeData.expanded && treeData.expanded  == true\">\n                <ul style=\"list-style-type: none;\">\n                    <li style=\"cursor: pointer\" *ngFor=\"let leaf of treeData.children\">\n                        <div>\n                            <span [ngClass]=\"{'glyphicon glyphicon-minus': leaf.expanded, 'glyphicon glyphicon-plus': (!leaf.expanded && leaf.children)}\" (click)=\"toggle(leaf)\"> </span>\n                            <img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\"><label (click)=\"emitData(leaf)\">{{ leaf.text }}</label>\n                        </div>\n                        <div *ngIf=\"leaf.expanded && leaf.expanded  == true\">\n                            <tree-view [dataTableBindData]=\"leaf\" [dataReader]=\"'children'\" (selectedRecord)=\"setSelectedRecord($event)\"></tree-view>\n                        </div>\n                    </li>\n                </ul>\n\n            </div>\n        </li>\n    </ul>\n    ",
        providers: [treeview_service_1.TreeViewService]
    }),
    __metadata("design:paramtypes", [treeview_service_1.TreeViewService])
], TreeViewComponent);
exports.TreeViewComponent = TreeViewComponent;
//# sourceMappingURL=treeview.component.js.map